package com.laihua.projecthelper

import com.laihua.projecthelper.implement.CodeFileFindResult
import com.laihua.projecthelper.implement.FileCalc
import java.io.File

/**
 * Author: pengjunjing
 * Date: 2022/3/7
 * Description:单文件引用工具类,用于抽取到对应模块的时候使用
 */
fun main(args: Array<String>) {
    println("开始读取文件")
    val projectRoot = File("")
    val projectRootF = File(projectRoot.absolutePath)

    println(projectRoot.absolutePath)
    //要检查资源的目录
    val baseFile = File("${projectRoot.absolutePath}${File.separator}laihuaBase${File.separator}src${File.separator}")
    //从项目根目录开始查找
//    val baseFile = File("${projectRoot.absolutePath}")
    println(baseFile.absolutePath + "\n")
    val readImageRes = ReadUtil().readImageRes(baseFile)
    val readCodeFile = ReadUtil().readCodeFile(projectRootF)

    startFind(readImageRes, readCodeFile)
}

fun startFind(readImageRes: MutableList<File>, readCodeFile: MutableList<File>) {
    //统计所有有调用到资源的代码文件
    val codeFileList = mutableListOf<CodeFileFindResult>()
    readCodeFile.forEach {
        val parseFile = ParseUtil().parseFile(it)
        if (parseFile.isNotEmpty()) {
            codeFileList.add(CodeFileFindResult(it, parseFile))
        }

    }

    val resList: List<FileCalc> = readImageRes.map {
        FileCalc(it)
    }

    //统计引用次数
    codeFileList.forEach { codeFile: CodeFileFindResult ->//代码文件
        codeFile.codeLine.forEach { codeLine: String ->//一行代码
            resList.forEach { resFile: FileCalc ->//资源文件
                if (codeLine.contains(resFile.implementName, ignoreCase = true)) {
                    //引用次数+1
                    resFile.implementCode.add(codeFile)
                }
            }
        }
    }

    resList.forEach { fileCalc ->
        //计算所有依赖是属于哪个模块的
        fileCalc.calcImplementModule()
    }

    //只有一个依赖的列表
    val singleList = mutableListOf<FileCalc>()
    resList.forEach { fileCalc ->

        if (fileCalc.implementModule.size == 1) {
            //只有一个模块依赖的该文件,移动到
            singleList.add(fileCalc)
        }
    }

    //按模块排序
    singleList.sortBy {
        it.implementModule.toList()[0]
    }

    singleList.forEach {
        val implementModuleName = it.implementModule.toList()[0]
        if (it.belongModule != null && implementModuleName.lowercase() != it.belongModule.lowercase()) {
            println("模块:$implementModuleName,依赖了 ${it.belongModule} 模块下的文件: ${it.file.name}")
        }
    }

}

